# Generated by roxygen2: do not edit by hand

export(clone)
export(dropallfactors)
export(microtable)
export(tidy_taxonomy)
export(trans_abund)
export(trans_alpha)
export(trans_beta)
export(trans_classifier)
export(trans_diff)
export(trans_env)
export(trans_func)
export(trans_metab)
export(trans_network)
export(trans_norm)
export(trans_nullmodel)
export(trans_venn)
import(RColorBrewer)
import(ape)
import(data.table)
import(ggplot2)
import(grid)
import(scales)
import(vegan)
importFrom(R6,R6Class)
importFrom(grDevices,colorRampPalette)
importFrom(igraph,ecount)
importFrom(lifecycle,deprecated)
importFrom(magrittr,"%<>%")
importFrom(magrittr,"%>%")
importFrom(reshape2,acast)
importFrom(rlang,"!!")
importFrom(rlang,sym)
importFrom(stats,fisher.test)
importFrom(stats,median)
importFrom(tibble,rownames_to_column)
