#' @import ape
#' @import vegan
#' @import data.table
#' @import ggplot2
#' @import grid
#' @import scales
#' @import RColorBrewer
#' @importFrom magrittr %<>%
#' @importFrom magrittr %>%
#' @importFrom R6 R6Class
#' @importFrom tibble rownames_to_column
#' @importFrom rlang !!
#' @importFrom rlang sym
#' @importFrom stats fisher.test
#' @importFrom stats median
#' @importFrom reshape2 acast
#' @importFrom grDevices colorRampPalette
#' @importFrom igraph ecount
#' @importFrom lifecycle deprecated
NULL
