% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{prok_func_NJC19_list}
\alias{prok_func_NJC19_list}
\title{The NJC19 database}
\format{
list
}
\usage{
data(prok_func_NJC19_list)
}
\description{
The NJC19 database
}
\keyword{datasets}
